/*
 * Galatea Dialog Manager:
 * (c)2003 Takuya NISHIMOTO (nishi@hil.t.u-tokyo.ac.jp)
 * $Id: IInteractionController.java,v 1.3 2007/08/09 02:31:03 nishi Exp $
 */

package galatea.dialog;

import galatea.document.ContentState;
import galatea.document.StateMap;
import galatea.io.OutItemQueueManager;

public interface IInteractionController
{
	public boolean waitDevicesForReady();
	public void setStateMap(StateMap stateMap) throws Exception;
	public void mainLoop() throws Exception;
    public String getNextDocFile();
    public void resetNextDocFile();
	public void dispDocInfo(String file, String state, String src, String trans);
	public String getPostString();
}
